@@ -21,11 +21,12 @@ import { Delete, DeleteForever, FilterList } from "@material-ui/icons";
 import React, { useCallback, useEffect, useState } from "react";
 import { useDispatch } from "react-redux";
 import { useHistory } from "react-router";
-import { toggleSnackbar } from "../../../actions";
+import { toggleSnackbar } from "../../../redux/explorer";
 import API from "../../../middleware/Api";
 import { sizeToString } from "../../../utils";
 import FileFilter from "../Dialogs/FileFilter";
 import { formatLocalTime } from "../../../utils/datetime";
+import Chip from "@material-ui/core/Chip";
 
 const useStyles = makeStyles((theme) => ({
     root: {
@@ -70,6 +71,10 @@ const useStyles = makeStyles((theme) => ({
         top: 20,
         width: 1,
     },
+    disabledBadge: {
+        marginLeft: theme.spacing(1),
+        height: 18,
+    },
 }));
 
 export default function File() {
@@ -374,7 +379,7 @@ export default function File() {
                                     上传者
                                 </TableCell>
                                 <TableCell style={{ minWidth: 150 }}>
-                                    上传于
+                                    创建于
                                 </TableCell>
                                 <TableCell style={{ minWidth: 100 }}>
                                     操作
@@ -408,6 +413,15 @@ export default function File() {
                                             }
                                         >
                                             {row.Name}
+                                            {row.UploadSessionID && (
+                                                <Chip
+                                                    className={
+                                                        classes.disabledBadge
+                                                    }
+                                                    size="small"
+                                                    label="上传中"
+                                                />
+                                            )}
                                         </Link>
                                     </TableCell>
                                     <TableCell align={"right"}>
