@@ -21,7 +21,7 @@ import Alert from "@material-ui/lab/Alert";
 import React, { useCallback, useEffect, useState } from "react";
 import { useDispatch } from "react-redux";
 import { useHistory } from "react-router";
-import { toggleSnackbar } from "../../../actions";
+import { toggleSnackbar } from "../../../redux/explorer";
 import API from "../../../middleware/Api";
 import PathSelector from "../../FileManager/PathSelector";
 
