@@ -15,11 +15,12 @@ import Typography from "@material-ui/core/Typography";
 import React, { useCallback, useState } from "react";
 import { useDispatch } from "react-redux";
 import { useHistory } from "react-router";
-import { toggleSnackbar } from "../../../../actions";
+import { toggleSnackbar } from "../../../../redux/explorer";
 import API from "../../../../middleware/Api";
 import DomainInput from "../../Common/DomainInput";
 import SizeInput from "../../Common/SizeInput";
 import MagicVar from "../../Dialogs/MagicVar";
+import { getNumber } from "../../../../utils";
 
 const useStyles = makeStyles((theme) => ({
     stepContent: {
@@ -75,7 +76,7 @@ const steps = [
         optional: false,
     },
     {
-        title: "上传限制",
+        title: "上传设置",
         optional: false,
     },
     {
@@ -108,6 +109,7 @@ export default function LocalGuide(props) {
                   MaxSize: "0",
                   OptionsSerialized: {
                       file_type: "",
+                      chunk_size: 25 << 20,
                   },
               }
     );
@@ -176,6 +178,11 @@ export default function LocalGuide(props) {
         policyCopy.IsOriginLinkEnable =
             policyCopy.IsOriginLinkEnable === "true";
         policyCopy.MaxSize = parseInt(policyCopy.MaxSize);
+        policyCopy.OptionsSerialized.chunk_size = parseInt(
+            policyCopy.OptionsSerialized.chunk_size
+        );
+        policyCopy.OptionsSerialized.placeholder_with_size =
+            policyCopy.OptionsSerialized.placeholder_with_size === "true";
         policyCopy.IsPrivate = policyCopy.IsPrivate === "true";
         policyCopy.OptionsSerialized.file_type = policyCopy.OptionsSerialized.file_type.split(
             ","
@@ -678,6 +685,34 @@ export default function LocalGuide(props) {
                         </div>
                     </Collapse>
 
+                    <div className={classes.subStepContainer}>
+                        <div className={classes.stepNumberContainer}>
+                            <div className={classes.stepNumber}>
+                                {getNumber(3, [
+                                    policy.MaxSize !== "0",
+                                    policy.OptionsSerialized.file_type !== "",
+                                ])}
+                            </div>
+                        </div>
+                        <div className={classes.subStepContent}>
+                            <Typography variant={"body2"}>
+                                请指定分片上传时的分片大小，填写为 0
+                                表示不使用分片上传。
+                                <br />
+                                启用分片上传后，用户上传的文件将会被切分成分片逐个上传到存储端，当上传中断后，用户可以选择从上次上传的分片后继续开始上传。
+                            </Typography>
+                            <div className={classes.form}>
+                                <SizeInput
+                                    value={policy.OptionsSerialized.chunk_size}
+                                    onChange={handleOptionChange("chunk_size")}
+                                    min={0}
+                                    max={9223372036854775807}
+                                    label={"分片上传大小"}
+                                />
+                            </div>
+                        </div>
+                    </div>
+
                     <div className={classes.stepFooter}>
                         <Button
                             color={"default"}
