@@ -5,7 +5,6 @@ import StepLabel from "@material-ui/core/StepLabel";
 import Step from "@material-ui/core/Step";
 import Typography from "@material-ui/core/Typography";
 import { useDispatch } from "react-redux";
-import { toggleSnackbar } from "../../../../actions";
 import Link from "@material-ui/core/Link";
 import FormControl from "@material-ui/core/FormControl";
 import InputLabel from "@material-ui/core/InputLabel";
@@ -24,6 +23,7 @@ import { getNumber } from "../../../../utils";
 import Autocomplete from "@material-ui/lab/Autocomplete";
 import TextField from "@material-ui/core/TextField";
 import AlertDialog from "../../Dialogs/Alert";
+import { toggleSnackbar } from "../../../../redux/explorer";
 
 const useStyles = makeStyles((theme) => ({
     stepContent: {
@@ -172,6 +172,8 @@ export default function S3Guide(props) {
                   OptionsSerialized: {
                       file_type: "",
                       region: "us-east-2",
+                      chunk_size: 25 << 20,
+                      placeholder_with_size: "false",
                   },
               }
     );
@@ -224,6 +226,11 @@ export default function S3Guide(props) {
             policyCopy.IsOriginLinkEnable === "true";
         policyCopy.IsPrivate = policyCopy.IsPrivate === "true";
         policyCopy.MaxSize = parseInt(policyCopy.MaxSize);
+        policyCopy.OptionsSerialized.chunk_size = parseInt(
+            policyCopy.OptionsSerialized.chunk_size
+        );
+        policyCopy.OptionsSerialized.placeholder_with_size =
+            policyCopy.OptionsSerialized.placeholder_with_size === "true";
         policyCopy.OptionsSerialized.file_type = policyCopy.OptionsSerialized.file_type.split(
             ","
         );
@@ -984,6 +991,80 @@ export default function S3Guide(props) {
                         </div>
                     </Collapse>
 
+                    <div className={classes.subStepContainer}>
+                        <div className={classes.stepNumberContainer}>
+                            <div className={classes.stepNumber}>
+                                {getNumber(3, [
+                                    policy.MaxSize !== "0",
+                                    policy.OptionsSerialized.file_type !== "",
+                                ])}
+                            </div>
+                        </div>
+                        <div className={classes.subStepContent}>
+                            <Typography variant={"body2"}>
+                                请指定分片上传时的分片大小，范围 5 MB ~ 5 GB。
+                                <br />
+                                启用分片上传后，用户上传的文件将会被切分成分片逐个上传到存储端，当上传中断后，用户可以选择从上次上传的分片后继续开始上传。
+                            </Typography>
+                            <div className={classes.form}>
+                                <SizeInput
+                                    value={policy.OptionsSerialized.chunk_size}
+                                    onChange={handleOptionChange("chunk_size")}
+                                    min={5 << 20}
+                                    max={53687091200}
+                                    label={"分片上传大小"}
+                                />
+                            </div>
+                        </div>
+                    </div>
+
+                    <div className={classes.subStepContainer}>
+                        <div className={classes.stepNumberContainer}>
+                            <div className={classes.stepNumber}>
+                                {getNumber(4, [
+                                    policy.MaxSize !== "0",
+                                    policy.OptionsSerialized.file_type !== "",
+                                ])}
+                            </div>
+                        </div>
+                        <div className={classes.subStepContent}>
+                            <Typography variant={"body2"}>
+                                是否要再用户开始上传时就创建占位符文件并扣除用户容量？开启后，可以防止用户恶意发起多个上传请求但不完成上传。
+                            </Typography>
+                            <div className={classes.form}>
+                                <FormControl required component="fieldset">
+                                    <RadioGroup
+                                        aria-label="gender"
+                                        name="gender1"
+                                        value={
+                                            policy.OptionsSerialized
+                                                .placeholder_with_size
+                                        }
+                                        onChange={handleOptionChange(
+                                            "placeholder_with_size"
+                                        )}
+                                        row
+                                    >
+                                        <FormControlLabel
+                                            value={"true"}
+                                            control={
+                                                <Radio color={"primary"} />
+                                            }
+                                            label="创建占位符文件"
+                                        />
+                                        <FormControlLabel
+                                            value={"false"}
+                                            control={
+                                                <Radio color={"primary"} />
+                                            }
+                                            label="不创建"
+                                        />
+                                    </RadioGroup>
+                                </FormControl>
+                            </div>
+                        </div>
+                    </div>
+
                     <div className={classes.stepFooter}>
                         <Button
                             color={"default"}
