@@ -23,15 +23,23 @@ import PropTypes from "prop-types";
 import React, { Component } from "react";
 import { connect } from "react-redux";
 import { withRouter } from "react-router-dom";
+import { isCompressFile, isPreviewable, isTorrent } from "../../config";
+import Auth from "../../middleware/Auth";
+import { allowSharePreview } from "../../utils/index";
+import pathHelper from "../../utils/page";
+import RefreshIcon from "@material-ui/icons/Refresh";
 import {
-    openCompressDialog,
-    openCreateFileDialog,
-    refreshFileList,
-} from "../../actions";
+    openPreview,
+    setSelectedTarget,
+    startBatchDownload,
+    toggleObjectInfoSidebar,
+} from "../../redux/explorer/action";
 import {
     changeContextMenu,
     navigateTo,
+    openCompressDialog,
     openCopyDialog,
+    openCreateFileDialog,
     openCreateFolderDialog,
     openDecompressDialog,
     openGetSourceDialog,
@@ -43,21 +51,16 @@ import {
     openRenameDialog,
     openShareDialog,
     openTorrentDownloadDialog,
+    refreshFileList,
     setNavigatorLoadingStatus,
-    setSelectedTarget,
     showImgPreivew,
     toggleSnackbar,
-} from "../../actions/index";
-import { isCompressFile, isPreviewable, isTorrent } from "../../config";
-import Auth from "../../middleware/Auth";
-import { allowSharePreview } from "../../utils/index";
-import pathHelper from "../../utils/page";
-import RefreshIcon from "@material-ui/icons/Refresh";
-import { openPreview } from "../../actions";
+} from "../../redux/explorer";
+import { pathJoin } from "../Uploader/core/utils";
 import {
-    setSideBar,
-    toggleObjectInfoSidebar,
-} from "../../redux/explorer/action";
+    openFileSelector,
+    openFolderSelector,
+} from "../../redux/viewUpdate/action";
 
 const styles = () => ({
     propover: {},
@@ -157,6 +160,15 @@ const mapDispatchToProps = (dispatch) => {
         toggleObjectInfoSidebar: (open) => {
             dispatch(toggleObjectInfoSidebar(open));
         },
+        startBatchDownload: (share) => {
+            dispatch(startBatchDownload(share));
+        },
+        openFileSelector: () => {
+            dispatch(openFileSelector());
+        },
+        openFolderSelector: () => {
+            dispatch(openFolderSelector());
+        },
     };
 };
 
@@ -176,8 +188,7 @@ class ContextMenuCompoment extends Component {
     };
 
     openArchiveDownload = () => {
-        this.props.changeContextMenu("file", false);
-        this.props.openLoadingDialog("打包中...");
+        this.props.startBatchDownload(this.props.share);
     };
 
     openDownload = () => {
@@ -197,27 +208,10 @@ class ContextMenuCompoment extends Component {
 
     enterFolder = () => {
         this.props.navigateTo(
-            this.props.path === "/"
-                ? this.props.path + this.props.selected[0].name
-                : this.props.path + "/" + this.props.selected[0].name
+            pathJoin([this.props.path, this.props.selected[0].name])
         );
     };
 
-    clickUpload = (id) => {
-        this.props.changeContextMenu("empty", false);
-        const uploadButton = document.getElementsByClassName(id)[0];
-        if (document.body.contains(uploadButton)) {
-            uploadButton.click();
-        } else {
-            this.props.toggleSnackbar(
-                "top",
-                "right",
-                "上传组件还未加载完成",
-                "warning"
-            );
-        }
-    };
-
     // 暂时只对空白处右键菜单使用这个函数，疑似有bug会导致的一个菜单被默认选中。
     // 相关issue： https://github.com/mui-org/material-ui/issues/23747
     renderMenuItems = (items) => {
@@ -273,13 +267,13 @@ class ContextMenuCompoment extends Component {
             center: [
                 {
                     condition: true,
-                    onClick: () => this.clickUpload("uploadFileForm"),
+                    onClick: () => this.props.openFileSelector(),
                     icon: <UploadIcon />,
                     text: "上传文件",
                 },
                 {
                     condition: true,
-                    onClick: () => this.clickUpload("uploadFolderForm"),
+                    onClick: () => this.props.openFolderSelector(),
                     icon: <FolderUpload />,
                     text: "上传目录",
                 },
@@ -345,9 +339,7 @@ class ContextMenuCompoment extends Component {
                             <Divider className={classes.divider} />
                             <MenuItem
                                 dense
-                                onClick={() =>
-                                    this.clickUpload("uploadFileForm")
-                                }
+                                onClick={() => this.props.openFileSelector()}
                             >
                                 <StyledListItemIcon>
                                     <UploadIcon />
@@ -358,9 +350,7 @@ class ContextMenuCompoment extends Component {
                             </MenuItem>
                             <MenuItem
                                 dense
-                                onClick={() =>
-                                    this.clickUpload("uploadFolderForm")
-                                }
+                                onClick={() => this.props.openFolderSelector()}
                             >
                                 <StyledListItemIcon>
                                     <FolderUpload />
@@ -472,28 +462,25 @@ class ContextMenuCompoment extends Component {
                                 </div>
                             )}
 
-                            {(this.props.isMultiple || this.props.withFolder) &&
-                                (user.group.allowArchiveDownload ||
-                                    !isHomePage) && (
-                                    <MenuItem
-                                        dense
-                                        onClick={() =>
-                                            this.openArchiveDownload()
-                                        }
-                                    >
-                                        <StyledListItemIcon>
-                                            <DownloadIcon />
-                                        </StyledListItemIcon>
-                                        <Typography variant="inherit">
-                                            打包下载
-                                        </Typography>
-                                    </MenuItem>
-                                )}
+                            {(this.props.isMultiple ||
+                                this.props.withFolder) && (
+                                <MenuItem
+                                    dense
+                                    onClick={() => this.openArchiveDownload()}
+                                >
+                                    <StyledListItemIcon>
+                                        <DownloadIcon />
+                                    </StyledListItemIcon>
+                                    <Typography variant="inherit">
+                                        打包下载
+                                    </Typography>
+                                </MenuItem>
+                            )}
 
                             {!this.props.isMultiple &&
                                 this.props.withFile &&
                                 isHomePage &&
-                                user.policy.allowSource && (
+                                this.props.selected[0].source_enabled && (
                                     <MenuItem
                                         dense
                                         onClick={() =>
