@@ -11,27 +11,19 @@ import {
     withStyles,
 } from "@material-ui/core";
 import TableSortLabel from "@material-ui/core/TableSortLabel";
-import SadIcon from "@material-ui/icons/SentimentVeryDissatisfied";
-import EmptyIcon from "@material-ui/icons/Unarchive";
 import classNames from "classnames";
 import PropTypes from "prop-types";
 import React, { Component } from "react";
 import { configure, GlobalHotKeys } from "react-hotkeys";
 import { connect } from "react-redux";
 import { withRouter } from "react-router-dom";
-import {
-    changeContextMenu,
-    navigateTo,
-    navigateUp,
-    openRemoveDialog,
-    setSelectedTarget,
-} from "../../actions/index";
-import explorer from "../../redux/explorer";
+import explorer, { changeContextMenu, navigateTo, navigateUp, openRemoveDialog, setSelectedTarget } from "../../redux/explorer";
 import { isMac } from "../../utils";
 import pathHelper from "../../utils/page";
 import ContextMenu from "./ContextMenu";
 import ImgPreivew from "./ImgPreview";
 import ObjectIcon from "./ObjectIcon";
+import Nothing from "../Placeholder/Nothing";
 
 const styles = (theme) => ({
     paper: {
@@ -80,25 +72,6 @@ const styles = (theme) => ({
     errorMsg: {
         marginTop: "10px",
     },
-    emptyContainer: {
-        bottom: "0",
-        height: "300px",
-        margin: "50px auto",
-        width: "300px",
-        color: theme.palette.text.disabled,
-        textAlign: "center",
-        paddingTop: "20px",
-    },
-    emptyIcon: {
-        fontSize: "160px",
-    },
-    emptyInfoBig: {
-        fontSize: "25px",
-        color: theme.palette.text.disabled,
-    },
-    emptyInfoSmall: {
-        color: theme.palette.text.hint,
-    },
     hideAuto: {
         [theme.breakpoints.down("sm")]: {
             display: "none",
@@ -131,6 +104,18 @@ const styles = (theme) => ({
         top: 20,
         width: 1,
     },
+    gridContainer: {
+        [theme.breakpoints.down("sm")]: {
+            gridTemplateColumns: "repeat(auto-fill,minmax(180px,1fr))",
+        },
+        [theme.breakpoints.up("md")]: {
+            gridTemplateColumns: "repeat(auto-fill,minmax(220px,1fr))",
+        },
+        display: "grid",
+    },
+    gridItem: {
+        flex: "1 1 220px",
+    },
 });
 
 const mapStateToProps = (state) => {
@@ -306,25 +291,26 @@ class ExplorerCompoment extends Component {
                         <TableCell className={classes.hideAuto}>
                             <TableSortLabel
                                 active={
-                                    this.props.sortMethod === "timePos" ||
-                                    this.props.sortMethod === "timeRev"
+                                    this.props.sortMethod === "modifyTimePos" ||
+                                    this.props.sortMethod === "modifyTimeRev"
                                 }
                                 direction={
-                                    this.props.sortMethod === "timePos"
+                                    this.props.sortMethod === "modifyTimePos"
                                         ? "asc"
                                         : "des"
                                 }
                                 onClick={() => {
                                     this.props.changeSort(
-                                        this.props.sortMethod === "timePos"
-                                            ? "timeRev"
-                                            : "timePos"
+                                        this.props.sortMethod ===
+                                            "modifyTimePos"
+                                            ? "modifyTimeRev"
+                                            : "modifyTimePos"
                                     );
                                 }}
                             >
-                                日期
-                                {this.props.sortMethod === "timePos" ||
-                                this.props.sortMethod === "timeRev" ? (
+                                修改日期
+                                {this.props.sortMethod === "modifyTimePos" ||
+                                this.props.sortMethod === "modifyTimeRev" ? (
                                     <span className={classes.visuallyHidden}>
                                         {this.props.sortMethod === "sizeRes"
                                             ? "sorted descending"
@@ -370,15 +356,13 @@ class ExplorerCompoment extends Component {
                             container
                             spacing={0}
                             alignItems="flex-start"
+                            className={classes.gridContainer}
                         >
                             {this.props.dirList.map((value, index) => (
                                 <Grid
                                     key={value.id}
                                     item
-                                    xs={6}
-                                    md={3}
-                                    sm={4}
-                                    lg={2}
+                                    className={classes.gridItem}
                                 >
                                     <ObjectIcon
                                         key={value.id}
@@ -404,15 +388,13 @@ class ExplorerCompoment extends Component {
                             container
                             spacing={0}
                             alignItems="flex-start"
+                            className={classes.gridContainer}
                         >
                             {this.props.fileList.map((value, index) => (
                                 <Grid
+                                    className={classes.gridItem}
                                     key={value.id}
                                     item
-                                    xs={6}
-                                    md={3}
-                                    sm={4}
-                                    lg={2}
                                 >
                                     <ObjectIcon
                                         key={value.id}
@@ -469,15 +451,10 @@ class ExplorerCompoment extends Component {
                     this.props.fileList.length === 0 &&
                     !this.props.loading &&
                     !this.props.navigatorError && (
-                        <div className={classes.emptyContainer}>
-                            <EmptyIcon className={classes.emptyIcon} />
-                            <div className={classes.emptyInfoBig}>
-                                拖拽文件至此
-                            </div>
-                            <div className={classes.emptyInfoSmall}>
-                                或点击右下方“上传文件”按钮添加文件
-                            </div>
-                        </div>
+                        <Nothing
+                            primary={"拖拽文件至此"}
+                            secondary={"或点击右下方“上传文件”按钮添加文件"}
+                        />
                     )}
                 {((this.props.keywords !== "" &&
                     this.props.dirList.length === 0 &&
@@ -488,14 +465,7 @@ class ExplorerCompoment extends Component {
                         this.props.fileList.length === 0 &&
                         !this.props.loading &&
                         !this.props.navigatorError &&
-                        !isHomePage)) && (
-                    <div className={classes.emptyContainer}>
-                        <SadIcon className={classes.emptyIcon} />
-                        <div className={classes.emptyInfoBig}>
-                            什么都没有找到
-                        </div>
-                    </div>
-                )}
+                        !isHomePage)) && <Nothing primary={"什么都没有找到"} />}
                 {showView && view}
             </div>
         );
