@@ -1,26 +1,18 @@
 import React, { Component } from "react";
 import PropTypes from "prop-types";
 import { connect } from "react-redux";
-import {
-    closeAllModals,
-    toggleSnackbar,
-    setModalsLoading,
-    refreshFileList,
-    refreshStorage,
-    openLoadingDialog,
-} from "../../actions/index";
 import PathSelector from "./PathSelector";
 import API, { baseURL } from "../../middleware/Api";
 import {
-    withStyles,
     Button,
-    TextField,
+    CircularProgress,
     Dialog,
     DialogActions,
     DialogContent,
-    DialogTitle,
     DialogContentText,
-    CircularProgress,
+    DialogTitle,
+    TextField,
+    withStyles,
 } from "@material-ui/core";
 import Loading from "../Modals/Loading";
 import CopyDialog from "../Modals/Copy";
@@ -29,6 +21,16 @@ import { withRouter } from "react-router-dom";
 import pathHelper from "../../utils/page";
 import DecompressDialog from "../Modals/Decompress";
 import CompressDialog from "../Modals/Compress";
+import {
+    closeAllModals,
+    openLoadingDialog,
+    refreshFileList,
+    refreshStorage,
+    setModalsLoading,
+    toggleSnackbar,
+} from "../../redux/explorer";
+import OptionSelector from "../Modals/OptionSelector";
+import { getDownloadURL } from "../../services/file";
 
 const styles = (theme) => ({
     wrapper: {
@@ -120,17 +122,7 @@ class ModalsCompoment extends Component {
         if (this.props.loading !== nextProps.loading) {
             // 打包下载
             if (nextProps.loading === true) {
-                if (nextProps.loadingText === "打包中...") {
-                    if (
-                        pathHelper.isSharePage(this.props.location.pathname) &&
-                        this.props.share &&
-                        this.props.share.score > 0
-                    ) {
-                        this.scoreHandler(this.archiveDownload);
-                        return;
-                    }
-                    this.archiveDownload();
-                } else if (nextProps.loadingText === "获取下载地址...") {
+                if (nextProps.loadingText === "获取下载地址...") {
                     if (
                         pathHelper.isSharePage(this.props.location.pathname) &&
                         this.props.share &&
@@ -178,24 +170,7 @@ class ModalsCompoment extends Component {
     };
 
     Download = () => {
-        let reqURL = "";
-        if (this.props.selected[0].key) {
-            const downloadPath =
-                this.props.selected[0].path === "/"
-                    ? this.props.selected[0].path + this.props.selected[0].name
-                    : this.props.selected[0].path +
-                      "/" +
-                      this.props.selected[0].name;
-            reqURL =
-                "/share/download/" +
-                this.props.selected[0].key +
-                "?path=" +
-                encodeURIComponent(downloadPath);
-        } else {
-            reqURL = "/file/download/" + this.props.selected[0].id;
-        }
-
-        API.put(reqURL)
+        getDownloadURL(this.props.selected[0])
             .then((response) => {
                 window.location.assign(response.data);
                 this.onClose();
@@ -212,55 +187,6 @@ class ModalsCompoment extends Component {
             });
     };
 
-    archiveDownload = () => {
-        const dirs = [],
-            items = [];
-        this.props.selected.map((value) => {
-            if (value.type === "dir") {
-                dirs.push(value.id);
-            } else {
-                items.push(value.id);
-            }
-            return null;
-        });
-
-        let reqURL = "/file/archive";
-        const postBody = {
-            items: items,
-            dirs: dirs,
-        };
-        if (pathHelper.isSharePage(this.props.location.pathname)) {
-            reqURL = "/share/archive/" + window.shareInfo.key;
-            postBody["path"] = this.props.selected[0].path;
-        }
-
-        API.post(reqURL, postBody)
-            .then((response) => {
-                if (response.rawData.code === 0) {
-                    this.onClose();
-                    window.location.assign(response.data);
-                } else {
-                    this.props.toggleSnackbar(
-                        "top",
-                        "right",
-                        response.rawData.msg,
-                        "warning"
-                    );
-                }
-                this.onClose();
-                this.props.refreshStorage();
-            })
-            .catch((error) => {
-                this.props.toggleSnackbar(
-                    "top",
-                    "right",
-                    error.message,
-                    "error"
-                );
-                this.onClose();
-            });
-    };
-
     submitRemove = (e) => {
         e.preventDefault();
         this.props.setModalsLoading(true);
@@ -631,6 +557,7 @@ class ModalsCompoment extends Component {
         return (
             <div>
                 <Loading />
+                <OptionSelector />
                 <Dialog
                     open={this.props.modalsStatus.getSource}
                     onClose={this.onClose}
