@@ -7,40 +7,18 @@ import MusicNote from "@material-ui/icons/MusicNote";
 import BackIcon from "@material-ui/icons/ArrowBack";
 import OpenIcon from "@material-ui/icons/OpenInNew";
 import DownloadIcon from "@material-ui/icons/CloudDownload";
-import OpenFolderIcon from "@material-ui/icons/FolderOpen";
 import RenameIcon from "@material-ui/icons/BorderColor";
 import MoveIcon from "@material-ui/icons/Input";
 import DeleteIcon from "@material-ui/icons/Delete";
-import SaveIcon from "@material-ui/icons/Save";
 import MenuIcon from "@material-ui/icons/Menu";
 import { isPreviewable } from "../../config";
-import {
-    drawerToggleAction,
-    setSelectedTarget,
-    navigateTo,
-    openCreateFolderDialog,
-    changeContextMenu,
-    searchMyFile,
-    saveFile,
-    openMusicDialog,
-    showImgPreivew,
-    toggleSnackbar,
-    openMoveDialog,
-    openRemoveDialog,
-    openShareDialog,
-    openRenameDialog,
-    openLoadingDialog,
-    setSessionStatus,
-    openPreview,
-    audioPreviewSetIsOpen,
-} from "../../actions";
 import {
     allowSharePreview,
-    checkGetParameters,
     changeThemeColor,
+    sizeToString,
+    vhCheck,
 } from "../../utils";
-import Uploader from "../Upload/Uploader.js";
-import { sizeToString, vhCheck } from "../../utils";
+import Uploader from "../Uploader/Uploader.js";
 import pathHelper from "../../utils/page";
 import SezrchBar from "./SearchBar";
 import StorageBar from "./StorageBar";
@@ -50,20 +28,20 @@ import { AccountArrowRight, AccountPlus, LogoutVariant } from "mdi-material-ui";
 import { withRouter } from "react-router-dom";
 import {
     AppBar,
-    Toolbar,
-    Typography,
-    withStyles,
-    withTheme,
     Drawer,
-    SwipeableDrawer,
-    IconButton,
+    Grow,
     Hidden,
+    IconButton,
+    List,
     ListItem,
     ListItemIcon,
     ListItemText,
-    List,
-    Grow,
+    SwipeableDrawer,
+    Toolbar,
     Tooltip,
+    Typography,
+    withStyles,
+    withTheme,
 } from "@material-ui/core";
 import Auth from "../../middleware/Auth";
 import API from "../../middleware/Api";
@@ -71,6 +49,27 @@ import FileTag from "./FileTags";
 import { Assignment, Devices, MoreHoriz, Settings } from "@material-ui/icons";
 import Divider from "@material-ui/core/Divider";
 import SubActions from "../FileManager/Navigator/SubActions";
+import {
+    audioPreviewSetIsOpen,
+    changeContextMenu,
+    drawerToggleAction,
+    navigateTo,
+    openCreateFolderDialog,
+    openLoadingDialog,
+    openMoveDialog,
+    openMusicDialog,
+    openPreview,
+    openRemoveDialog,
+    openRenameDialog,
+    openShareDialog,
+    saveFile,
+    searchMyFile,
+    setSelectedTarget,
+    setSessionStatus,
+    showImgPreivew,
+    toggleSnackbar,
+} from "../../redux/explorer";
+import { startBatchDownload } from "../../redux/explorer/action";
 
 vhCheck();
 const drawerWidth = 240;
@@ -89,8 +88,8 @@ const mapStateToProps = (state) => {
         subTitle: state.viewUpdate.subTitle,
         loadUploader: state.viewUpdate.loadUploader,
         isLogin: state.viewUpdate.isLogin,
-        audioPreviewPlayingName:state.explorer.audioPreview.playingName,
-        audioPreviewIsOpen:state.explorer.audioPreview.isOpen,
+        audioPreviewPlayingName: state.explorer.audioPreview.playingName,
+        audioPreviewIsOpen: state.explorer.audioPreview.isOpen,
     };
 };
 
@@ -150,6 +149,9 @@ const mapDispatchToProps = (dispatch) => {
         audioPreviewOpen: () => {
             dispatch(audioPreviewSetIsOpen(true));
         },
+        startBatchDownload: (share) => {
+            dispatch(startBatchDownload(share));
+        },
     };
 };
 
@@ -182,7 +184,7 @@ const styles = (theme) => ({
     },
     menuButtonDesktop: {
         marginRight: 20,
-        [theme.breakpoints.down("sm")]: {
+        [theme.breakpoints.down("xs")]: {
             display: "none",
         },
     },
@@ -198,6 +200,12 @@ const styles = (theme) => ({
     },
     upDrawer: {
         overflowX: "hidden",
+        [theme.breakpoints.up("sm")]: {
+            display: "flex",
+            flexDirection: "column",
+            height: "100%",
+            justifyContent: "space-between",
+        },
     },
     drawerOpen: {
         width: drawerWidth,
@@ -278,13 +286,6 @@ const styles = (theme) => ({
     },
     minStickDrawer: {
         overflowY: "auto",
-        [theme.breakpoints.up("sm")]: {
-            height: "calc(var(--vh, 100vh) - 145px)",
-        },
-
-        [theme.breakpoints.down("sm")]: {
-            minHeight: "calc(var(--vh, 100vh) - 360px)",
-        },
     },
 });
 class NavbarCompoment extends Component {
@@ -316,16 +317,11 @@ class NavbarCompoment extends Component {
 
     UNSAFE_componentWillReceiveProps = (nextProps) => {
         if (
-            (this.props.selected.length <= 1 &&
-                !(!this.props.isMultiple && this.props.withFile)) !==
-            (nextProps.selected.length <= 1 &&
-                !(!nextProps.isMultiple && nextProps.withFile))
+            (this.props.selected.length === 0) !==
+            (nextProps.selected.length === 0)
         ) {
             changeThemeColor(
-                !(
-                    this.props.selected.length <= 1 &&
-                    !(!this.props.isMultiple && this.props.withFile)
-                )
+                !(this.props.selected.length === 0)
                     ? this.props.theme.palette.type === "dark"
                         ? this.props.theme.palette.background.default
                         : this.props.theme.palette.primary.main
@@ -338,18 +334,6 @@ class NavbarCompoment extends Component {
         this.setState((state) => ({ mobileOpen: !state.mobileOpen }));
     };
 
-    loadUploader = () => {
-        if (pathHelper.isHomePage(this.props.location.pathname)) {
-            return (
-                <>
-                    {this.props.loadUploader && this.props.isLogin && (
-                        <Uploader />
-                    )}
-                </>
-            );
-        }
-    };
-
     openDownload = () => {
         if (!allowSharePreview()) {
             this.props.toggleSnackbar(
@@ -364,7 +348,7 @@ class NavbarCompoment extends Component {
     };
 
     archiveDownload = () => {
-        this.props.openLoadingDialog("打包中...");
+        this.props.startBatchDownload(this.props.share);
     };
 
     signOut = () => {
@@ -469,43 +453,42 @@ class NavbarCompoment extends Component {
                                     </ListItemIcon>
                                     <ListItemText primary="任务队列" />
                                 </ListItem>
+                                {pathHelper.isMobile() && (
+                                    <>
+                                        <Divider />
+                                        <ListItem
+                                            button
+                                            key="个人设置"
+                                            onClick={() =>
+                                                this.props.history.push(
+                                                    "/setting?"
+                                                )
+                                            }
+                                        >
+                                            <ListItemIcon>
+                                                <Settings
+                                                    className={classes.iconFix}
+                                                />
+                                            </ListItemIcon>
+                                            <ListItemText primary="个人设置" />
+                                        </ListItem>
+
+                                        <ListItem
+                                            button
+                                            key="退出登录"
+                                            onClick={this.signOut}
+                                        >
+                                            <ListItemIcon>
+                                                <LogoutVariant
+                                                    className={classes.iconFix}
+                                                />
+                                            </ListItemIcon>
+                                            <ListItemText primary="退出登录" />
+                                        </ListItem>
+                                    </>
+                                )}
                             </List>
                         </div>
-
-                        {pathHelper.isMobile() && (
-                            <>
-                                <Divider />
-                                <List>
-                                    <ListItem
-                                        button
-                                        key="个人设置"
-                                        onClick={() =>
-                                            this.props.history.push("/setting?")
-                                        }
-                                    >
-                                        <ListItemIcon>
-                                            <Settings
-                                                className={classes.iconFix}
-                                            />
-                                        </ListItemIcon>
-                                        <ListItemText primary="个人设置" />
-                                    </ListItem>
-
-                                    <ListItem
-                                        button
-                                        key="退出登录"
-                                        onClick={this.signOut}
-                                    >
-                                        <ListItemIcon>
-                                            <LogoutVariant
-                                                className={classes.iconFix}
-                                            />
-                                        </ListItemIcon>
-                                        <ListItemText primary="退出登录" />
-                                    </ListItem>
-                                </List>
-                            </>
-                        )}
                         <div>
                             <StorageBar></StorageBar>
                         </div>
@@ -549,56 +532,42 @@ class NavbarCompoment extends Component {
                     className={classes.appBar}
                     color={
                         this.props.theme.palette.type !== "dark" &&
-                        this.props.selected.length <= 1 &&
-                        !(!this.props.isMultiple && this.props.withFile)
+                        this.props.selected.length === 0
                             ? "primary"
                             : "default"
                     }
                 >
                     <Toolbar>
-                        {this.props.selected.length <= 1 &&
-                            !(
-                                !this.props.isMultiple && this.props.withFile
-                            ) && (
-                                <IconButton
-                                    color="inherit"
-                                    aria-label="Open drawer"
-                                    onClick={this.handleDrawerToggle}
-                                    className={classes.menuButton}
-                                >
-                                    <MenuIcon />
-                                </IconButton>
-                            )}
-                        {this.props.selected.length <= 1 &&
-                            !(
-                                !this.props.isMultiple && this.props.withFile
-                            ) && (
-                                <IconButton
-                                    color="inherit"
-                                    aria-label="Open drawer"
-                                    onClick={() =>
-                                        this.props.handleDesktopToggle(
-                                            !this.props.desktopOpen
-                                        )
-                                    }
-                                    className={classes.menuButtonDesktop}
-                                >
-                                    <MenuIcon />
-                                </IconButton>
-                            )}
-                        {(this.props.selected.length > 1 ||
-                            (!this.props.isMultiple && this.props.withFile)) &&
+                        {this.props.selected.length === 0 && (
+                            <IconButton
+                                color="inherit"
+                                aria-label="Open drawer"
+                                onClick={this.handleDrawerToggle}
+                                className={classes.menuButton}
+                            >
+                                <MenuIcon />
+                            </IconButton>
+                        )}
+                        {this.props.selected.length === 0 && (
+                            <IconButton
+                                color="inherit"
+                                aria-label="Open drawer"
+                                onClick={() =>
+                                    this.props.handleDesktopToggle(
+                                        !this.props.desktopOpen
+                                    )
+                                }
+                                className={classes.menuButtonDesktop}
+                            >
+                                <MenuIcon />
+                            </IconButton>
+                        )}
+                        {this.props.selected.length > 0 &&
                             (isHomePage ||
                                 pathHelper.isSharePage(
                                     this.props.location.pathname
                                 )) && (
-                                <Grow
-                                    in={
-                                        this.props.selected.length > 1 ||
-                                        (!this.props.isMultiple &&
-                                            this.props.withFile)
-                                    }
-                                >
+                                <Grow in={this.props.selected.length > 0}>
                                     <IconButton
                                         color="inherit"
                                         className={classes.menuIcon}
@@ -610,33 +579,31 @@ class NavbarCompoment extends Component {
                                     </IconButton>
                                 </Grow>
                             )}
-                        {this.props.selected.length <= 1 &&
-                            !(
-                                !this.props.isMultiple && this.props.withFile
-                            ) && (
-                                <Typography
-                                    variant="h6"
-                                    color="inherit"
-                                    noWrap
-                                    onClick={() => {
-                                        this.props.history.push("/");
-                                    }}
-                                >
-                                    {this.props.subTitle
-                                        ? this.props.subTitle
-                                        : this.props.title}
-                                </Typography>
-                            )}
+                        {this.props.selected.length === 0 && (
+                            <Typography
+                                variant="h6"
+                                color="inherit"
+                                noWrap
+                                onClick={() => {
+                                    this.props.history.push("/");
+                                }}
+                            >
+                                {this.props.subTitle
+                                    ? this.props.subTitle
+                                    : this.props.title}
+                            </Typography>
+                        )}
 
                         {!this.props.isMultiple &&
-                            this.props.withFile &&
+                            (this.props.withFile || this.props.withFolder) &&
                             !pathHelper.isMobile() && (
                                 <Typography variant="h6" color="inherit" noWrap>
                                     {this.props.selected[0].name}{" "}
-                                    {(isHomePage ||
-                                        pathHelper.isSharePage(
-                                            this.props.location.pathname
-                                        )) &&
+                                    {this.props.withFile &&
+                                        (isHomePage ||
+                                            pathHelper.isSharePage(
+                                                this.props.location.pathname
+                                            )) &&
                                         "(" +
                                             sizeToString(
                                                 this.props.selected[0].size
@@ -648,37 +615,12 @@ class NavbarCompoment extends Component {
                         {this.props.selected.length > 1 &&
                             !pathHelper.isMobile() && (
                                 <Typography variant="h6" color="inherit" noWrap>
-                                    {this.props.selected.length}个对象
+                                    {this.props.selected.length} 个对象
                                 </Typography>
                             )}
-                        {this.props.selected.length <= 1 &&
-                            !(
-                                !this.props.isMultiple && this.props.withFile
-                            ) && <SezrchBar />}
+                        {this.props.selected.length === 0 && <SezrchBar />}
                         <div className={classes.grow} />
-                        {(this.props.selected.length > 1 ||
-                            (!this.props.isMultiple && this.props.withFile)) &&
-                            !isHomePage &&
-                            !pathHelper.isSharePage(
-                                this.props.location.pathname
-                            ) &&
-                            Auth.Check(this.props.isLogin) &&
-                            !checkGetParameters("share") && (
-                                <div className={classes.sectionForFile}>
-                                    <Tooltip title="保存">
-                                        <IconButton
-                                            color="inherit"
-                                            onClick={() =>
-                                                this.props.saveFile()
-                                            }
-                                        >
-                                            <SaveIcon />
-                                        </IconButton>
-                                    </Tooltip>
-                                </div>
-                            )}
-                        {(this.props.selected.length > 1 ||
-                            (!this.props.isMultiple && this.props.withFile)) &&
+                        {this.props.selected.length > 0 &&
                             (isHomePage || isSharePage) && (
                                 <div className={classes.sectionForFile}>
                                     {!this.props.isMultiple &&
@@ -729,67 +671,25 @@ class NavbarCompoment extends Component {
                                             </Grow>
                                         )}
                                     {(this.props.isMultiple ||
-                                        this.props.withFolder) &&
-                                        user.group.allowArchiveDownload && (
-                                            <Grow
-                                                in={
-                                                    (this.props.isMultiple ||
-                                                        this.props
-                                                            .withFolder) &&
-                                                    user.group
-                                                        .allowArchiveDownload
-                                                }
-                                            >
-                                                <Tooltip title="打包下载">
-                                                    <IconButton
-                                                        color="inherit"
-                                                        onClick={() =>
-                                                            this.archiveDownload()
-                                                        }
-                                                    >
-                                                        <DownloadIcon />
-                                                    </IconButton>
-                                                </Tooltip>
-                                            </Grow>
-                                        )}
-
-                                    {!this.props.isMultiple &&
-                                        this.props.withFolder && (
-                                            <Grow
-                                                in={
-                                                    !this.props.isMultiple &&
-                                                    this.props.withFolder
-                                                }
-                                            >
-                                                <Tooltip title="进入目录">
-                                                    <IconButton
-                                                        color="inherit"
-                                                        onClick={() =>
-                                                            this.props.navigateTo(
-                                                                this.props
-                                                                    .path ===
-                                                                    "/"
-                                                                    ? this.props
-                                                                          .path +
-                                                                          this
-                                                                              .props
-                                                                              .selected[0]
-                                                                              .name
-                                                                    : this.props
-                                                                          .path +
-                                                                          "/" +
-                                                                          this
-                                                                              .props
-                                                                              .selected[0]
-                                                                              .name
-                                                            )
-                                                        }
-                                                    >
-                                                        <OpenFolderIcon />
-                                                    </IconButton>
-                                                </Tooltip>
-                                            </Grow>
-                                        )}
+                                        this.props.withFolder) && (
+                                        <Grow
+                                            in={
+                                                this.props.isMultiple ||
+                                                this.props.withFolder
+                                            }
+                                        >
+                                            <Tooltip title="打包下载">
+                                                <IconButton
+                                                    color="inherit"
+                                                    onClick={() =>
+                                                        this.archiveDownload()
+                                                    }
+                                                >
+                                                    <DownloadIcon />
+                                                </IconButton>
+                                            </Tooltip>
+                                        </Grow>
+                                    )}
                                     {!this.props.isMultiple &&
                                         !pathHelper.isMobile() &&
                                         !isSharePage && (
@@ -888,31 +788,26 @@ class NavbarCompoment extends Component {
                                     )}
                                 </div>
                             )}
-                        {this.props.selected.length <= 1 &&
-                            !(
-                                !this.props.isMultiple && this.props.withFile
-                            ) && this.props.audioPreviewPlayingName!=null && (
-                            <IconButton
-                                title="音乐"
-                                className={classes.sideButton}
-                                onClick={this.props.audioPreviewOpen}
-                                color={"inherit"}
-                            >
-                                <MusicNote fontSize={ "default"} />
-                            </IconButton>
-                        )}
-
-                        {this.props.selected.length <= 1 &&
-                            !(
-                                !this.props.isMultiple && this.props.withFile
-                            ) && <UserAvatar />}
                         {this.props.selected.length <= 1 &&
                             !(!this.props.isMultiple && this.props.withFile) &&
+                            this.props.audioPreviewPlayingName != null && (
+                                <IconButton
+                                    title="音乐"
+                                    className={classes.sideButton}
+                                    onClick={this.props.audioPreviewOpen}
+                                    color={"inherit"}
+                                >
+                                    <MusicNote fontSize={"default"} />
+                                </IconButton>
+                            )}
+
+                        {this.props.selected.length === 0 && <UserAvatar />}
+                        {this.props.selected.length === 0 &&
                             isHomePage &&
                             pathHelper.isMobile() && <SubActions inherit />}
                     </Toolbar>
                 </AppBar>
-                {this.loadUploader()}
+                <Uploader />
 
                 <Hidden smUp implementation="css">
                     <SwipeableDrawer
