@@ -4,17 +4,17 @@ import StorageIcon from "@material-ui/icons/Storage";
 import { connect } from "react-redux";
 import API from "../../middleware/Api";
 import { sizeToString } from "../../utils";
-import { toggleSnackbar } from "../../actions";
 
 import {
-    withStyles,
-    LinearProgress,
-    Typography,
     Divider,
+    LinearProgress,
     Tooltip,
+    Typography,
+    withStyles,
 } from "@material-ui/core";
 import ButtonBase from "@material-ui/core/ButtonBase";
 import { withRouter } from "react-router";
+import { toggleSnackbar } from "../../redux/explorer";
 
 const mapStateToProps = (state) => {
     return {
@@ -68,6 +68,7 @@ const styles = (theme) => ({
     },
 });
 
+// TODO 使用 hooks 重构
 class StorageBarCompoment extends Component {
     state = {
         percent: 0,
