@@ -1,7 +1,6 @@
-import React, { useState, useCallback, useEffect } from "react";
+import React, { useCallback, useEffect, useState } from "react";
 import { makeStyles, Typography } from "@material-ui/core";
 import { useDispatch } from "react-redux";
-import { toggleSnackbar } from "../../actions";
 import Paper from "@material-ui/core/Paper";
 import Table from "@material-ui/core/Table";
 import TableHead from "@material-ui/core/TableHead";
@@ -12,6 +11,8 @@ import API from "../../middleware/Api";
 import { getTaskProgress, getTaskStatus, getTaskType } from "../../config";
 import Pagination from "@material-ui/lab/Pagination";
 import { formatLocalTime } from "../../utils/datetime";
+import { toggleSnackbar } from "../../redux/explorer";
+import Nothing from "../Placeholder/Nothing";
 
 const useStyles = makeStyles((theme) => ({
     layout: {
@@ -81,7 +82,7 @@ export default function Tasks() {
         }
         try {
             const res = JSON.parse(error);
-            return res.msg;
+            return `${res.msg}: ${res.error}`;
         } catch (e) {
             return "未知";
         }
@@ -140,6 +141,7 @@ export default function Tasks() {
                         ))}
                     </TableBody>
                 </Table>
+                {tasks.length === 0 && <Nothing primary={"没有记录"} />}
                 <div className={classes.footer}>
                     <Pagination
                         count={Math.ceil(total / 10)}
