@@ -1,5 +1,6 @@
 import { ThunkAction } from "redux-thunk";
 import { AnyAction } from "redux";
+
 export interface ActionSetSubtitle extends AnyAction {
     type: "SET_SUBTITLE";
     title: string;
@@ -18,6 +19,14 @@ export const closeContextMenu = () => {
     };
 };
 
+export const changeContextMenu = (type: string, open: boolean) => {
+    return {
+        type: "CHANGE_CONTEXT_MENU",
+        menuType: type,
+        open: open,
+    };
+};
+
 export const changeSubTitle = (
     title: string
 ): ThunkAction<any, any, any, any> => {
@@ -30,3 +39,22 @@ export const changeSubTitle = (
         dispatch(setSubtitle(title));
     };
 };
+
+export const setOptionModal = (option: any) => {
+    return {
+        type: "SET_OPTION_MODAL",
+        option: option,
+    };
+};
+
+export const openFileSelector = () => {
+    return {
+        type: "OPEN_FILE_SELECTOR",
+    };
+};
+
+export const openFolderSelector = () => {
+    return {
+        type: "OPEN_FOLDER_SELECTOR",
+    };
+};
