@@ -3,6 +3,8 @@ export interface CloudreveFile {
     name: string;
     size: number;
     date: string;
+    create_date: string;
+    path: string;
     type: "up" | "file" | "dir";
 }
 
@@ -12,4 +14,11 @@ export type SortMethod =
     | "namePos"
     | "nameRev"
     | "timePos"
-    | "timeRev";
+    | "timeRev"
+    | "modifyTimePos"
+    | "modifyTimeRev";
+
+export enum BatchDownloadMethod {
+    ClientStream,
+    ServerArchive,
+}
