@@ -149,6 +149,9 @@ export function pathJoin(parts, sep) {
 }
 
 export function basename(path) {
+    if (!path) {
+        return "";
+    }
     const pathList = path.split("/");
     pathList.pop();
     return pathList.join("/") === "" ? "/" : pathList.join("/");
@@ -159,6 +162,10 @@ export function filename(path) {
     return pathList.pop();
 }
 
+export function fileNameNoExt(filename) {
+    return filename.substring(0, filename.lastIndexOf(".")) || filename;
+}
+
 export function randomStr(length) {
     let result = "";
     const characters =
@@ -185,6 +192,13 @@ export const isMac = () => {
     return navigator.platform.toUpperCase().indexOf("MAC") >= 0;
 };
 
+export const isMobileSafari = () => {
+    const ua = window.navigator.userAgent;
+    const iOS = !!ua.match(/iPad/i) || !!ua.match(/iPhone/i);
+    const webkit = !!ua.match(/WebKit/i);
+    return iOS && webkit && !ua.match(/CriOS/i);
+};
+
 export function vhCheck() {
     const vh = window.innerHeight;
     document.documentElement.style.setProperty("--vh", `${vh}px`);
