@@ -1,7 +1,6 @@
 package onedrive
 
 import (
-	"bytes"
 	"context"
 	"encoding/json"
 	"errors"
@@ -18,7 +17,10 @@ import (
 
 	model "github.com/cloudreve/Cloudreve/v3/models"
 	"github.com/cloudreve/Cloudreve/v3/pkg/cache"
+	"github.com/cloudreve/Cloudreve/v3/pkg/filesystem/chunk"
+	"github.com/cloudreve/Cloudreve/v3/pkg/filesystem/chunk/backoff"
 	"github.com/cloudreve/Cloudreve/v3/pkg/filesystem/fsctx"
+	"github.com/cloudreve/Cloudreve/v3/pkg/mq"
 	"github.com/cloudreve/Cloudreve/v3/pkg/request"
 	"github.com/cloudreve/Cloudreve/v3/pkg/util"
 )
@@ -29,7 +31,8 @@ const (
 	// ChunkSize 服务端中转分片上传分片大小
 	ChunkSize uint64 = 10 * 1024 * 1024
 	// ListRetry 列取请求重试次数
-	ListRetry = 1
+	ListRetry       = 1
+	chunkRetrySleep = time.Second * 5
 )
 
 // GetSourcePath 获取文件的绝对路径
@@ -219,28 +222,21 @@ func (client *Client) GetUploadSessionStatus(ctx context.Context, uploadURL stri
 }
 
 // UploadChunk 上传分片
-func (client *Client) UploadChunk(ctx context.Context, uploadURL string, chunk *Chunk) (*UploadSessionResponse, error) {
+func (client *Client) UploadChunk(ctx context.Context, uploadURL string, content io.Reader, current *chunk.ChunkGroup) (*UploadSessionResponse, error) {
 	res, err := client.request(
-		ctx, "PUT", uploadURL, bytes.NewReader(chunk.Data[0:chunk.ChunkSize]),
-		request.WithContentLength(int64(chunk.ChunkSize)),
+		ctx, "PUT", uploadURL, content,
+		request.WithContentLength(current.Length()),
 		request.WithHeader(http.Header{
-			"Content-Range": {fmt.Sprintf("bytes %d-%d/%d", chunk.Offset, chunk.Offset+chunk.ChunkSize-1, chunk.Total)},
+			"Content-Range": {current.RangeHeader()},
 		}),
 		request.WithoutHeader([]string{"Authorization", "Content-Type"}),
 		request.WithTimeout(time.Duration(300)*time.Second),
 	)
 	if err != nil {
-		// 如果重试次数小于限制，5秒后重试
-		if chunk.Retried < model.GetIntSetting("onedrive_chunk_retries", 1) {
-			chunk.Retried++
-			util.Log().Debug("分片偏移%d上传失败[%s]，5秒钟后重试", chunk.Offset, err)
-			time.Sleep(time.Duration(5) * time.Second)
-			return client.UploadChunk(ctx, uploadURL, chunk)
-		}
-		return nil, err
+		return nil, fmt.Errorf("failed to upload OneDrive chunk #%d: %w", current.Index(), err)
 	}
 
-	if chunk.IsLast() {
+	if current.IsLast() {
 		return nil, nil
 	}
 
@@ -257,13 +253,17 @@ func (client *Client) UploadChunk(ctx context.Context, uploadURL string, chunk *
 }
 
 // Upload 上传文件
-func (client *Client) Upload(ctx context.Context, dst string, size int, file io.Reader) error {
+func (client *Client) Upload(ctx context.Context, file fsctx.FileHeader) error {
+	fileInfo := file.Info()
 	// 决定是否覆盖文件
-	overwrite := "replace"
-	if ctx.Value(fsctx.DisableOverwrite) != nil {
-		overwrite = "fail"
+	overwrite := "fail"
+	if fileInfo.Mode&fsctx.Overwrite == fsctx.Overwrite {
+		overwrite = "replace"
 	}
 
+	size := int(fileInfo.Size)
+	dst := fileInfo.SavePath
+
 	// 小文件，使用简单上传接口上传
 	if size <= int(SmallFileSize) {
 		_, err := client.SimpleUpload(ctx, dst, file, int64(size), WithConflictBehavior(overwrite))
@@ -277,46 +277,24 @@ func (client *Client) Upload(ctx context.Context, dst string, size int, file io.
 		return err
 	}
 
-	offset := 0
-	chunkNum := size / int(ChunkSize)
-	if size%int(ChunkSize) != 0 {
-		chunkNum++
+	// Initial chunk groups
+	chunks := chunk.NewChunkGroup(file, client.Policy.OptionsSerialized.ChunkSize, &backoff.ConstantBackoff{
+		Max:   model.GetIntSetting("chunk_retries", 5),
+		Sleep: chunkRetrySleep,
+	}, model.IsTrueVal(model.GetSettingByName("use_temp_chunk_buffer")))
+
+	uploadFunc := func(current *chunk.ChunkGroup, content io.Reader) error {
+		_, err := client.UploadChunk(ctx, uploadURL, content, current)
+		return err
 	}
 
-	chunkData := make([]byte, ChunkSize)
-
-	for i := 0; i < chunkNum; i++ {
-		select {
-		case <-ctx.Done():
-			util.Log().Debug("OneDrive 客户端取消")
-			return ErrClientCanceled
-		default:
-			// 分块
-			chunkSize := int(ChunkSize)
-			if size-offset < chunkSize {
-				chunkSize = size - offset
-			}
-
-			// 因为后面需要错误重试，这里要把分片内容读到内存中
-			chunkContent := chunkData[:chunkSize]
-			_, err := io.ReadFull(file, chunkContent)
-
-			chunk := Chunk{
-				Offset:    offset,
-				ChunkSize: chunkSize,
-				Total:     size,
-				Data:      chunkContent,
-			}
-
-			// 上传
-			_, err = client.UploadChunk(ctx, uploadURL, &chunk)
-			if err != nil {
-				return err
-			}
-			offset += chunkSize
+	// upload chunks
+	for chunks.Next() {
+		if err := chunks.Process(uploadFunc); err != nil {
+			return fmt.Errorf("failed to upload chunk #%d: %w", chunks.Index(), err)
 		}
-
 	}
+
 	return nil
 }
 
@@ -345,16 +323,6 @@ func (client *Client) SimpleUpload(ctx context.Context, dst string, body io.Read
 		request.WithTimeout(time.Duration(150)*time.Second),
 	)
 	if err != nil {
-		retried := 0
-		if v, ok := ctx.Value(fsctx.RetryCtx).(int); ok {
-			retried = v
-		}
-		if retried < model.GetIntSetting("onedrive_chunk_retries", 1) {
-			retried++
-			util.Log().Debug("文件[%s]上传失败[%s]，5秒钟后重试", dst, err)
-			time.Sleep(time.Duration(5) * time.Second)
-			return client.SimpleUpload(context.WithValue(ctx, fsctx.RetryCtx, retried), dst, body, size, opts...)
-		}
 		return nil, err
 	}
 
@@ -483,9 +451,9 @@ func (client *Client) GetThumbURL(ctx context.Context, dst string, w, h uint) (s
 // MonitorUpload 监控客户端分片上传进度
 func (client *Client) MonitorUpload(uploadURL, callbackKey, path string, size uint64, ttl int64) {
 	// 回调完成通知chan
-	callbackChan := make(chan bool)
-	callbackSignal.Store(callbackKey, callbackChan)
-	defer callbackSignal.Delete(callbackKey)
+	callbackChan := mq.GlobalMQ.Subscribe(callbackKey, 1)
+	defer mq.GlobalMQ.Unsubscribe(callbackKey, callbackChan)
+
 	timeout := model.GetIntSetting("onedrive_monitor_timeout", 600)
 	interval := model.GetIntSetting("onedrive_callback_check", 20)
 
@@ -510,16 +478,16 @@ func (client *Client) MonitorUpload(uploadURL, callbackKey, path string, size ui
 				if resErr, ok := err.(*RespError); ok {
 					if resErr.APIError.Code == "itemNotFound" {
 						util.Log().Debug("上传会话已完成，稍后检查回调")
-						time.Sleep(time.Duration(interval) * time.Second)
-						util.Log().Debug("开始检查回调")
-						_, ok := cache.Get("callback_" + callbackKey)
-						if ok {
+						select {
+						case <-time.After(time.Duration(interval) * time.Second):
 							util.Log().Warning("未发送回调，删除文件")
 							cache.Deletes([]string{callbackKey}, "callback_")
 							_, err = client.Delete(context.Background(), []string{path})
 							if err != nil {
 								util.Log().Warning("无法删除未回调的文件，%s", err)
 							}
+						case <-callbackChan:
+							util.Log().Debug("客户端完成回调")
 						}
 						return
 					}
@@ -556,15 +524,6 @@ func (client *Client) MonitorUpload(uploadURL, callbackKey, path string, size ui
 	}
 }
 
-// FinishCallback 向Monitor发送回调结束信号
-func FinishCallback(key string) {
-	if signal, ok := callbackSignal.Load(key); ok {
-		if signalChan, ok := signal.(chan bool); ok {
-			close(signalChan)
-		}
-	}
-}
-
 func sysError(err error) *RespError {
 	return &RespError{APIError: APIError{
 		Code:    "system",
