@@ -3,6 +3,8 @@ package filesystem
 import (
 	"context"
 	"errors"
+	"github.com/cloudreve/Cloudreve/v3/pkg/filesystem/response"
+	testMock "github.com/stretchr/testify/mock"
 	"os"
 	"testing"
 
@@ -11,12 +13,10 @@ import (
 	"github.com/cloudreve/Cloudreve/v3/pkg/cache"
 	"github.com/cloudreve/Cloudreve/v3/pkg/conf"
 	"github.com/cloudreve/Cloudreve/v3/pkg/filesystem/fsctx"
-	"github.com/cloudreve/Cloudreve/v3/pkg/filesystem/response"
 	"github.com/cloudreve/Cloudreve/v3/pkg/serializer"
 	"github.com/cloudreve/Cloudreve/v3/pkg/util"
 	"github.com/jinzhu/gorm"
 	"github.com/stretchr/testify/assert"
-	testMock "github.com/stretchr/testify/mock"
 )
 
 func TestFileSystem_ListPhysical(t *testing.T) {
@@ -214,7 +214,7 @@ func TestFileSystem_CreateDirectory(t *testing.T) {
 	asserts.Equal(ErrFileExisted, err)
 	asserts.NoError(mock.ExpectationsWereMet())
 
-	// 存在同名目录
+	// 存在同名目录，直接返回
 	mock.ExpectQuery("SELECT(.+)").
 		WithArgs(1).
 		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}).AddRow(1, 1))
@@ -225,13 +225,37 @@ func TestFileSystem_CreateDirectory(t *testing.T) {
 
 	mock.ExpectQuery("SELECT(.+)files").WillReturnRows(sqlmock.NewRows([]string{"id", "name"}))
 	mock.ExpectBegin()
-	mock.ExpectExec("INSERT(.+)").WillReturnError(errors.New("s"))
-	mock.ExpectRollback()
-	_, err = fs.CreateDirectory(ctx, "/ad/ab")
-	asserts.Error(err)
+	// ab
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs("ab", 2, 1).
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}).AddRow(3, 1))
+	mock.ExpectCommit()
+	res, err := fs.CreateDirectory(ctx, "/ad/ab")
+	asserts.NoError(err)
+	asserts.EqualValues(3, res.ID)
 	asserts.NoError(mock.ExpectationsWereMet())
 
 	// 成功创建
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs(1).
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}).AddRow(1, 1))
+	// ad
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs(1, 1, "ad").
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}).AddRow(2, 1))
+
+	mock.ExpectQuery("SELECT(.+)files").WillReturnRows(sqlmock.NewRows([]string{"id", "name"}))
+	mock.ExpectBegin()
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs("ab", 2, 1).
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}))
+	mock.ExpectExec("INSERT(.+)").WillReturnResult(sqlmock.NewResult(1, 1))
+	mock.ExpectCommit()
+	_, err = fs.CreateDirectory(ctx, "/ad/ab")
+	asserts.NoError(err)
+	asserts.NoError(mock.ExpectationsWereMet())
+
+	// 成功创建, 递归创建父目录
 	// 根目录
 	mock.ExpectQuery("SELECT(.+)").
 		WithArgs(1).
@@ -239,20 +263,62 @@ func TestFileSystem_CreateDirectory(t *testing.T) {
 	// ad
 	mock.ExpectQuery("SELECT(.+)").
 		WithArgs(1, 1, "ad").
-		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}).AddRow(2, 1))
-
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}))
+	// 根目录
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs(1).
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}).AddRow(1, 1))
 	mock.ExpectQuery("SELECT(.+)files").WillReturnRows(sqlmock.NewRows([]string{"id", "name"}))
+	// 创建ad
 	mock.ExpectBegin()
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs("ad", 1, 1).
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}))
+	mock.ExpectExec("INSERT(.+)").WillReturnResult(sqlmock.NewResult(2, 1))
+	mock.ExpectCommit()
+	mock.ExpectQuery("SELECT(.+)files").WillReturnRows(sqlmock.NewRows([]string{"id", "name"}))
+	// 创建ab
+	mock.ExpectBegin()
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs("ab", 2, 1).
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}))
 	mock.ExpectExec("INSERT(.+)").WillReturnResult(sqlmock.NewResult(1, 1))
 	mock.ExpectCommit()
 	_, err = fs.CreateDirectory(ctx, "/ad/ab")
 	asserts.NoError(err)
 	asserts.NoError(mock.ExpectationsWereMet())
 
-	// 父目录不存在
-	mock.ExpectQuery("SELECT(.+)folders").WillReturnRows(sqlmock.NewRows([]string{"id", "name"}))
-	_, err = fs.CreateDirectory(ctx, "/ad")
-	asserts.Equal(ErrRootProtected, err)
+	// 底层创建失败
+	// 成功创建
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs(1).
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}).AddRow(1, 1))
+	// ad
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs(1, 1, "ad").
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}))
+	// 根目录
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs(1).
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}).AddRow(1, 1))
+	mock.ExpectQuery("SELECT(.+)files").WillReturnRows(sqlmock.NewRows([]string{"id", "name"}))
+	// 创建ad
+	mock.ExpectBegin()
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs("ad", 1, 1).
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}))
+	mock.ExpectExec("INSERT(.+)").WillReturnResult(sqlmock.NewResult(2, 1)).WillReturnError(errors.New("error"))
+	mock.ExpectRollback()
+	_, err = fs.CreateDirectory(ctx, "/ad/ab")
+	asserts.Error(err)
+	asserts.NoError(mock.ExpectationsWereMet())
+
+	// 直接创建根目录
+	mock.ExpectQuery("SELECT(.+)").
+		WithArgs(1).
+		WillReturnRows(sqlmock.NewRows([]string{"id", "owner_id"}).AddRow(1, 1))
+	_, err = fs.CreateDirectory(ctx, "/")
+	asserts.NoError(err)
 	asserts.NoError(mock.ExpectationsWereMet())
 }
 
@@ -296,12 +362,12 @@ func TestFileSystem_ListDeleteDirs(t *testing.T) {
 	{
 		mock.ExpectQuery("SELECT(.+)").
 			WillReturnRows(
-				sqlmock.NewRows([]string{"id"}).
-					AddRow(1).
-					AddRow(2).
-					AddRow(3),
+				sqlmock.NewRows([]string{"id", "parent_id"}).
+					AddRow(1, 0).
+					AddRow(2, 0).
+					AddRow(3, 0),
 			)
-		mock.ExpectQuery("SELECT(.+)").
+		mock.ExpectQuery("SELECT(.+)files(.+)").
 			WithArgs(1, 2, 3).
 			WillReturnRows(
 				sqlmock.NewRows([]string{"id", "name"}).
@@ -316,21 +382,47 @@ func TestFileSystem_ListDeleteDirs(t *testing.T) {
 		asserts.NoError(mock.ExpectationsWereMet())
 	}
 
+	// 成功,忽略根目录
+	{
+		mock.ExpectQuery("SELECT(.+)").
+			WillReturnRows(
+				sqlmock.NewRows([]string{"id", "parent_id"}).
+					AddRow(1, 0).
+					AddRow(2, nil).
+					AddRow(3, 0),
+			)
+		mock.ExpectQuery("SELECT(.+)files(.+)").
+			WithArgs(1, 3).
+			WillReturnRows(
+				sqlmock.NewRows([]string{"id", "name"}).
+					AddRow(4, "1.txt").
+					AddRow(5, "2.txt").
+					AddRow(6, "3.txt"),
+			)
+		fs.CleanTargets()
+		err := fs.ListDeleteDirs(context.Background(), []uint{1})
+		asserts.NoError(err)
+		asserts.Len(fs.FileTarget, 3)
+		asserts.Len(fs.DirTarget, 2)
+		asserts.NoError(mock.ExpectationsWereMet())
+	}
+
 	// 检索文件发生错误
 	{
 		mock.ExpectQuery("SELECT(.+)").
 			WillReturnRows(
-				sqlmock.NewRows([]string{"id"}).
-					AddRow(1).
-					AddRow(2).
-					AddRow(3),
+				sqlmock.NewRows([]string{"id", "parent_id"}).
+					AddRow(1, 0).
+					AddRow(2, 0).
+					AddRow(3, 0),
 			)
 		mock.ExpectQuery("SELECT(.+)").
 			WithArgs(1, 2, 3).
 			WillReturnError(errors.New("error"))
+		fs.CleanTargets()
 		err := fs.ListDeleteDirs(context.Background(), []uint{1})
 		asserts.Error(err)
-		asserts.Len(fs.DirTarget, 6)
+		asserts.Len(fs.DirTarget, 3)
 		asserts.NoError(mock.ExpectationsWereMet())
 	}
 	// 检索目录发生错误
@@ -349,7 +441,7 @@ func TestFileSystem_Delete(t *testing.T) {
 	cache.Set("pack_size_1", uint64(0), 0)
 	fs := &FileSystem{User: &model.User{
 		Model: gorm.Model{
-			ID: 1,
+			ID: 0,
 		},
 		Storage: 3,
 		Group:   model.Group{MaxStorage: 3},
@@ -361,10 +453,10 @@ func TestFileSystem_Delete(t *testing.T) {
 		fs.CleanTargets()
 		mock.ExpectQuery("SELECT(.+)").
 			WillReturnRows(
-				sqlmock.NewRows([]string{"id"}).
-					AddRow(1).
-					AddRow(2).
-					AddRow(3),
+				sqlmock.NewRows([]string{"id", "parent_id"}).
+					AddRow(1, 0).
+					AddRow(2, 0).
+					AddRow(3, 0),
 			)
 		mock.ExpectQuery("SELECT(.+)").
 			WithArgs(1, 2, 3).
@@ -380,18 +472,16 @@ func TestFileSystem_Delete(t *testing.T) {
 		// 删除文件记录
 		mock.ExpectBegin()
 		mock.ExpectExec("DELETE(.+)").
-			WillReturnResult(sqlmock.NewResult(0, 3))
+			WillReturnResult(sqlmock.NewResult(0, 1))
+		mock.ExpectExec("DELETE(.+)").
+			WillReturnResult(sqlmock.NewResult(0, 1))
+		mock.ExpectExec("UPDATE(.+)users(.+)storage(.+)").WillReturnResult(sqlmock.NewResult(1, 1))
 		mock.ExpectCommit()
 		// 删除对应分享
 		mock.ExpectBegin()
 		mock.ExpectExec("UPDATE(.+)shares").
 			WillReturnResult(sqlmock.NewResult(0, 3))
 		mock.ExpectCommit()
-		// 归还容量
-		mock.ExpectBegin()
-		mock.ExpectExec("UPDATE(.+)").
-			WillReturnResult(sqlmock.NewResult(0, 3))
-		mock.ExpectCommit()
 		// 删除目录
 		mock.ExpectBegin()
 		mock.ExpectExec("DELETE(.+)").
@@ -407,7 +497,6 @@ func TestFileSystem_Delete(t *testing.T) {
 		fs.DirTarget = []model.Folder{}
 		err := fs.Delete(ctx, []uint{1}, []uint{1}, true)
 		asserts.NoError(err)
-		asserts.Equal(uint64(0), fs.User.Storage)
 	}
 	//全部成功
 	{
@@ -419,10 +508,10 @@ func TestFileSystem_Delete(t *testing.T) {
 		asserts.NoError(err)
 		mock.ExpectQuery("SELECT(.+)").
 			WillReturnRows(
-				sqlmock.NewRows([]string{"id"}).
-					AddRow(1).
-					AddRow(2).
-					AddRow(3),
+				sqlmock.NewRows([]string{"id", "parent_id"}).
+					AddRow(1, 0).
+					AddRow(2, 0).
+					AddRow(3, 0),
 			)
 		mock.ExpectQuery("SELECT(.+)").
 			WithArgs(1, 2, 3).
@@ -438,18 +527,16 @@ func TestFileSystem_Delete(t *testing.T) {
 		// 删除文件记录
 		mock.ExpectBegin()
 		mock.ExpectExec("DELETE(.+)").
-			WillReturnResult(sqlmock.NewResult(0, 3))
+			WillReturnResult(sqlmock.NewResult(0, 1))
+		mock.ExpectExec("DELETE(.+)").
+			WillReturnResult(sqlmock.NewResult(0, 1))
+		mock.ExpectExec("UPDATE(.+)users(.+)storage(.+)").WillReturnResult(sqlmock.NewResult(1, 1))
 		mock.ExpectCommit()
 		// 删除对应分享
 		mock.ExpectBegin()
 		mock.ExpectExec("UPDATE(.+)shares").
 			WillReturnResult(sqlmock.NewResult(0, 3))
 		mock.ExpectCommit()
-		// 归还容量
-		mock.ExpectBegin()
-		mock.ExpectExec("UPDATE(.+)").
-			WillReturnResult(sqlmock.NewResult(0, 3))
-		mock.ExpectCommit()
 		// 删除目录
 		mock.ExpectBegin()
 		mock.ExpectExec("DELETE(.+)").
@@ -465,7 +552,6 @@ func TestFileSystem_Delete(t *testing.T) {
 		fs.DirTarget = []model.Folder{}
 		err = fs.Delete(ctx, []uint{1}, []uint{1}, false)
 		asserts.NoError(err)
-		asserts.Equal(uint64(0), fs.User.Storage)
 	}
 
 }
@@ -573,7 +659,9 @@ func TestFileSystem_Rename(t *testing.T) {
 		Model: gorm.Model{
 			ID: 1,
 		},
-	}}
+	},
+		Policy: &model.Policy{},
+	}
 	ctx := context.Background()
 
 	// 重命名文件 成功
@@ -683,7 +771,7 @@ func TestFileSystem_Rename(t *testing.T) {
 
 	// 新名字是文件，扩展名不合法
 	{
-		fs.User.Policy.OptionsSerialized.FileType = []string{"txt"}
+		fs.Policy.OptionsSerialized.FileType = []string{"txt"}
 		err := fs.Rename(ctx, []uint{}, []uint{10}, "1.jpg")
 		asserts.Error(err)
 		asserts.Equal(ErrIllegalObjectName, err)
@@ -691,7 +779,7 @@ func TestFileSystem_Rename(t *testing.T) {
 
 	// 新名字是目录，不应该检测扩展名
 	{
-		fs.User.Policy.OptionsSerialized.FileType = []string{"txt"}
+		fs.Policy.OptionsSerialized.FileType = []string{"txt"}
 		mock.ExpectQuery("SELECT(.+)folders(.+)").
 			WithArgs(10, 1).
 			WillReturnRows(sqlmock.NewRows([]string{"id", "name"}))
