@@ -19,7 +19,6 @@ import (
 
 	model "github.com/cloudreve/Cloudreve/v3/models"
 	"github.com/cloudreve/Cloudreve/v3/pkg/filesystem"
-	"github.com/cloudreve/Cloudreve/v3/pkg/filesystem/driver/local"
 	"github.com/cloudreve/Cloudreve/v3/pkg/filesystem/fsctx"
 	"github.com/cloudreve/Cloudreve/v3/pkg/util"
 )
@@ -32,6 +31,7 @@ type Handler struct {
 	// Logger is an optional error logger. If non-nil, it will be called
 	// for all HTTP requests.
 	Logger func(*http.Request, error)
+	Mutex  *sync.Mutex
 }
 
 func (h *Handler) stripPrefix(p string, uid uint) (string, int, error) {
@@ -62,13 +62,19 @@ func isPathExist(ctx context.Context, fs *filesystem.FileSystem, path string) (b
 
 func (h *Handler) ServeHTTP(w http.ResponseWriter, r *http.Request, fs *filesystem.FileSystem) {
 	status, err := http.StatusBadRequest, errUnsupportedMethod
+	h.Mutex.Lock()
 	if h.LockSystem == nil {
+		h.Mutex.Unlock()
 		status, err = http.StatusInternalServerError, errNoLockSystem
 	} else {
-		// 检查并新建LockSystem
-		if _, ok := h.LockSystem[fs.User.ID]; !ok {
+		// 检查并新建 LockSystem
+		ls, ok := h.LockSystem[fs.User.ID]
+		if !ok {
 			h.LockSystem[fs.User.ID] = NewMemLS()
+			ls = h.LockSystem[fs.User.ID]
 		}
+		h.Mutex.Unlock()
+
 		switch r.Method {
 		case "OPTIONS":
 			status, err = h.handleOptions(w, r, fs)
@@ -83,13 +89,13 @@ func (h *Handler) ServeHTTP(w http.ResponseWriter, r *http.Request, fs *filesyst
 		case "COPY", "MOVE":
 			status, err = h.handleCopyMove(w, r, fs)
 		case "LOCK":
-			status, err = h.handleLock(w, r, fs)
+			status, err = h.handleLock(w, r, fs, ls)
 		case "UNLOCK":
-			status, err = h.handleUnlock(w, r, fs)
+			status, err = h.handleUnlock(w, r, fs, ls)
 		case "PROPFIND":
-			status, err = h.handlePropfind(w, r, fs)
+			status, err = h.handlePropfind(w, r, fs, ls)
 		case "PROPPATCH":
-			status, err = h.handleProppatch(w, r, fs)
+			status, err = h.handleProppatch(w, r, fs, ls)
 		}
 	}
 
@@ -105,98 +111,111 @@ func (h *Handler) ServeHTTP(w http.ResponseWriter, r *http.Request, fs *filesyst
 }
 
 // OK
-func (h *Handler) lock(now time.Time, root string, fs *filesystem.FileSystem) (token string, status int, err error) {
-	token, err = h.LockSystem[fs.User.ID].Create(now, LockDetails{
-		Root:      root,
-		Duration:  infiniteTimeout,
-		ZeroDepth: true,
-	})
-	if err != nil {
-		if err == ErrLocked {
-			return "", StatusLocked, err
-		}
-		return "", http.StatusInternalServerError, err
-	}
-	return token, 0, nil
+func (h *Handler) lock(now time.Time, root string, fs *filesystem.FileSystem, ls LockSystem) (token string, status int, err error) {
+	//token, err = ls.Create(now, LockDetails{
+	//	Root:      root,
+	//	Duration:  infiniteTimeout,
+	//	ZeroDepth: true,
+	//})
+	//if err != nil {
+	//	if err == ErrLocked {
+	//		return "", StatusLocked, err
+	//	}
+	//	return "", http.StatusInternalServerError, err
+	//}
+
+	return fmt.Sprintf("%d", time.Now().Unix()), 0, nil
 }
 
 // ok
 func (h *Handler) confirmLocks(r *http.Request, src, dst string, fs *filesystem.FileSystem) (release func(), status int, err error) {
-	hdr := r.Header.Get("If")
-	if hdr == "" {
-		// An empty If header means that the client hasn't previously created locks.
-		// Even if this client doesn't care about locks, we still need to check that
-		// the resources aren't locked by another client, so we create temporary
-		// locks that would conflict with another client's locks. These temporary
-		// locks are unlocked at the end of the HTTP request.
-		now, srcToken, dstToken := time.Now(), "", ""
-		if src != "" {
-			srcToken, status, err = h.lock(now, src, fs)
-			if err != nil {
-				return nil, status, err
-			}
-		}
-		if dst != "" {
-			dstToken, status, err = h.lock(now, dst, fs)
-			if err != nil {
-				if srcToken != "" {
-					h.LockSystem[fs.User.ID].Unlock(now, srcToken)
-				}
-				return nil, status, err
-			}
-		}
 
-		return func() {
-			if dstToken != "" {
-				h.LockSystem[fs.User.ID].Unlock(now, dstToken)
-			}
-			if srcToken != "" {
-				h.LockSystem[fs.User.ID].Unlock(now, srcToken)
-			}
-		}, 0, nil
-	}
+	//hdr := r.Header.Get("If")
+	//h.Mutex.Lock()
+	//ls,ok := h.LockSystem[fs.User.ID]
+	//h.Mutex.Unlock()
+	//if !ok{
+	//	return nil, http.StatusInternalServerError, errNoLockSystem
+	//}
+	//
+	//if hdr == "" {
+	//	// An empty If header means that the client hasn't previously created locks.
+	//	// Even if this client doesn't care about locks, we still need to check that
+	//	// the resources aren't locked by another client, so we create temporary
+	//	// locks that would conflict with another client's locks. These temporary
+	//	// locks are unlocked at the end of the HTTP request.
+	//	now, srcToken, dstToken := time.Now(), "", ""
+	//	if src != "" {
+	//		srcToken, status, err = h.lock(now, src, fs,ls)
+	//		if err != nil {
+	//			return nil, status, err
+	//		}
+	//	}
+	//	if dst != "" {
+	//		dstToken, status, err = h.lock(now, dst, fs,ls)
+	//		if err != nil {
+	//			if srcToken != "" {
+	//				ls.Unlock(now, srcToken)
+	//			}
+	//			return nil, status, err
+	//		}
+	//	}
+	//
+	//	return func() {
+	//		if dstToken != "" {
+	//			ls.Unlock(now, dstToken)
+	//		}
+	//		if srcToken != "" {
+	//			ls.Unlock(now, srcToken)
+	//		}
+	//	}, 0, nil
+	//}
+	//
+	//ih, ok := parseIfHeader(hdr)
+	//if !ok {
+	//	return nil, http.StatusBadRequest, errInvalidIfHeader
+	//}
+	//// ih is a disjunction (OR) of ifLists, so any ifList will do.
+	//for _, l := range ih.lists {
+	//	lsrc := l.resourceTag
+	//	if lsrc == "" {
+	//		lsrc = src
+	//	} else {
+	//		u, err := url.Parse(lsrc)
+	//		if err != nil {
+	//			continue
+	//		}
+	//		//if u.Host != r.Host {
+	//		//	continue
+	//		//}
+	//		lsrc, status, err = h.stripPrefix(u.Path, fs.User.ID)
+	//		if err != nil {
+	//			return nil, status, err
+	//		}
+	//	}
+	//	release, err = ls.Confirm(
+	//		time.Now(),
+	//		lsrc,
+	//		dst,
+	//		l.conditions...,
+	//	)
+	//	if err == ErrConfirmationFailed {
+	//		continue
+	//	}
+	//	if err != nil {
+	//		return nil, http.StatusInternalServerError, err
+	//	}
+	//	return release, 0, nil
+	//}
+	//// Section 10.4.1 says that "If this header is evaluated and all state lists
+	//// fail, then the request must fail with a 412 (Precondition Failed) status."
+	//// We follow the spec even though the cond_put_corrupt_token test case from
+	//// the litmus test warns on seeing a 412 instead of a 423 (Locked).
+	//return nil, http.StatusPreconditionFailed, ErrLocked
 
-	ih, ok := parseIfHeader(hdr)
-	if !ok {
-		return nil, http.StatusBadRequest, errInvalidIfHeader
-	}
-	// ih is a disjunction (OR) of ifLists, so any ifList will do.
-	for _, l := range ih.lists {
-		lsrc := l.resourceTag
-		if lsrc == "" {
-			lsrc = src
-		} else {
-			u, err := url.Parse(lsrc)
-			if err != nil {
-				continue
-			}
-			//if u.Host != r.Host {
-			//	continue
-			//}
-			lsrc, status, err = h.stripPrefix(u.Path, fs.User.ID)
-			if err != nil {
-				return nil, status, err
-			}
-		}
-		release, err = h.LockSystem[fs.User.ID].Confirm(
-			time.Now(),
-			lsrc,
-			dst,
-			l.conditions...,
-		)
-		if err == ErrConfirmationFailed {
-			continue
-		}
-		if err != nil {
-			return nil, http.StatusInternalServerError, err
-		}
-		return release, 0, nil
-	}
-	// Section 10.4.1 says that "If this header is evaluated and all state lists
-	// fail, then the request must fail with a 412 (Precondition Failed) status."
-	// We follow the spec even though the cond_put_corrupt_token test case from
-	// the litmus test warns on seeing a 412 instead of a 423 (Locked).
-	return nil, http.StatusPreconditionFailed, ErrLocked
+	return func() {
+
+	}, 0, nil
 }
 
 //OK
@@ -247,7 +266,7 @@ func (h *Handler) handleGetHeadPost(w http.ResponseWriter, r *http.Request, fs *
 		return http.StatusInternalServerError, err
 	}
 
-	etag, err := findETag(ctx, fs, h.LockSystem[fs.User.ID], reqPath, &fs.FileTarget[0])
+	etag, err := findETag(ctx, fs, nil, reqPath, &fs.FileTarget[0])
 	if err != nil {
 		return http.StatusInternalServerError, err
 	}
@@ -273,6 +292,7 @@ func (h *Handler) handleDelete(w http.ResponseWriter, r *http.Request, fs *files
 	if err != nil {
 		return status, err
 	}
+
 	release, status, err := h.confirmLocks(r, reqPath, "", fs)
 	if err != nil {
 		return status, err
@@ -317,7 +337,6 @@ func (h *Handler) handlePut(w http.ResponseWriter, r *http.Request, fs *filesyst
 	defer cancel()
 	ctx = context.WithValue(ctx, fsctx.HTTPCtx, r.Context())
 	ctx = context.WithValue(ctx, fsctx.CancelFuncCtx, cancel)
-	ctx = context.WithValue(ctx, fsctx.ValidateCapacityOnceCtx, &sync.Once{})
 
 	fileSize, err := strconv.ParseUint(r.Header.Get("Content-Length"), 10, 64)
 	if err != nil {
@@ -325,7 +344,7 @@ func (h *Handler) handlePut(w http.ResponseWriter, r *http.Request, fs *filesyst
 	}
 	fileName := path.Base(reqPath)
 	filePath := path.Dir(reqPath)
-	fileData := local.FileStream{
+	fileData := fsctx.FileStream{
 		MIMEType:    r.Header.Get("Content-Type"),
 		File:        r.Body,
 		Size:        fileSize,
@@ -342,7 +361,7 @@ func (h *Handler) handlePut(w http.ResponseWriter, r *http.Request, fs *filesyst
 		fileList, err := model.RemoveFilesWithSoftLinks([]model.File{*originFile})
 		if err == nil && len(fileList) == 0 {
 			// 如果包含软连接，应重新生成新文件副本，并更新source_name
-			originFile.SourceName = fs.GenerateSavePath(ctx, fileData)
+			originFile.SourceName = fs.GenerateSavePath(ctx, &fileData)
 			fs.Use("AfterUpload", filesystem.HookUpdateSourceName)
 			fs.Use("AfterUploadCanceled", filesystem.HookUpdateSourceName)
 			fs.Use("AfterValidateFailed", filesystem.HookUpdateSourceName)
@@ -350,39 +369,33 @@ func (h *Handler) handlePut(w http.ResponseWriter, r *http.Request, fs *filesyst
 
 		fs.Use("BeforeUpload", filesystem.HookResetPolicy)
 		fs.Use("BeforeUpload", filesystem.HookValidateFile)
-		fs.Use("BeforeUpload", filesystem.HookChangeCapacity)
+		fs.Use("BeforeUpload", filesystem.HookValidateCapacityDiff)
 		fs.Use("AfterUploadCanceled", filesystem.HookCleanFileContent)
 		fs.Use("AfterUploadCanceled", filesystem.HookClearFileSize)
-		fs.Use("AfterUploadCanceled", filesystem.HookGiveBackCapacity)
 		fs.Use("AfterUploadCanceled", filesystem.HookCancelContext)
 		fs.Use("AfterUpload", filesystem.GenericAfterUpdate)
 		fs.Use("AfterValidateFailed", filesystem.HookCleanFileContent)
 		fs.Use("AfterValidateFailed", filesystem.HookClearFileSize)
-		fs.Use("AfterValidateFailed", filesystem.HookGiveBackCapacity)
 		ctx = context.WithValue(ctx, fsctx.FileModelCtx, *originFile)
+		fileData.Mode |= fsctx.Overwrite
 	} else {
 		// 给文件系统分配钩子
 		fs.Use("BeforeUpload", filesystem.HookValidateFile)
 		fs.Use("BeforeUpload", filesystem.HookValidateCapacity)
 		fs.Use("AfterUploadCanceled", filesystem.HookDeleteTempFile)
-		fs.Use("AfterUploadCanceled", filesystem.HookGiveBackCapacity)
 		fs.Use("AfterUploadCanceled", filesystem.HookCancelContext)
 		fs.Use("AfterUpload", filesystem.GenericAfterUpload)
+		fs.Use("AfterUpload", filesystem.HookGenerateThumb)
 		fs.Use("AfterValidateFailed", filesystem.HookDeleteTempFile)
-		fs.Use("AfterValidateFailed", filesystem.HookGiveBackCapacity)
-		fs.Use("AfterUploadFailed", filesystem.HookGiveBackCapacity)
-
-		// 禁止覆盖
-		ctx = context.WithValue(ctx, fsctx.DisableOverwrite, true)
 	}
 
 	// 执行上传
-	err = fs.Upload(ctx, fileData)
+	err = fs.Upload(ctx, &fileData)
 	if err != nil {
 		return http.StatusMethodNotAllowed, err
 	}
 
-	etag, err := findETag(ctx, fs, h.LockSystem[fs.User.ID], reqPath, &fs.FileTarget[0])
+	etag, err := findETag(ctx, fs, nil, reqPath, fileData.Model.(*model.File))
 	if err != nil {
 		return http.StatusInternalServerError, err
 	}
@@ -409,11 +422,7 @@ func (h *Handler) handleMkcol(w http.ResponseWriter, r *http.Request, fs *filesy
 	if r.ContentLength > 0 {
 		return http.StatusUnsupportedMediaType, nil
 	}
-	if strings.Contains(r.UserAgent(), "rclone") {
-		if _, ok := ctx.Value(fsctx.IgnoreDirectoryConflictCtx).(bool); !ok {
-			ctx = context.WithValue(ctx, fsctx.IgnoreDirectoryConflictCtx, true)
-		}
-	}
+
 	if _, err := fs.CreateDirectory(ctx, reqPath); err != nil {
 		return http.StatusConflict, err
 	}
@@ -507,130 +516,137 @@ func (h *Handler) handleCopyMove(w http.ResponseWriter, r *http.Request, fs *fil
 }
 
 // OK
-func (h *Handler) handleLock(w http.ResponseWriter, r *http.Request, fs *filesystem.FileSystem) (retStatus int, retErr error) {
+func (h *Handler) handleLock(w http.ResponseWriter, r *http.Request, fs *filesystem.FileSystem, ls LockSystem) (retStatus int, retErr error) {
 	defer fs.Recycle()
 
 	duration, err := parseTimeout(r.Header.Get("Timeout"))
 	if err != nil {
 		return http.StatusBadRequest, err
 	}
-	li, status, err := readLockInfo(r.Body)
+
+	reqPath, status, err := h.stripPrefix(r.URL.Path, fs.User.ID)
 	if err != nil {
 		return status, err
 	}
 
-	//ctx := r.Context()
-	token, ld, now, created := "", LockDetails{}, time.Now(), false
-	if li == (lockInfo{}) {
-		// An empty lockInfo means to refresh the lock.
-		ih, ok := parseIfHeader(r.Header.Get("If"))
-		if !ok {
-			return http.StatusBadRequest, errInvalidIfHeader
-		}
-		if len(ih.lists) == 1 && len(ih.lists[0].conditions) == 1 {
-			token = ih.lists[0].conditions[0].Token
-		}
-		if token == "" {
-			return http.StatusBadRequest, errInvalidLockToken
-		}
-		ld, err = h.LockSystem[fs.User.ID].Refresh(now, token, duration)
-		if err != nil {
-			if err == ErrNoSuchLock {
-				return http.StatusPreconditionFailed, err
-			}
-			return http.StatusInternalServerError, err
-		}
+	////ctx := r.Context()
+	//token, ld, now, created := "", LockDetails{}, time.Now(), false
+	//if li == (lockInfo{}) {
+	//	// An empty lockInfo means to refresh the lock.
+	//	ih, ok := parseIfHeader(r.Header.Get("If"))
+	//	if !ok {
+	//		return http.StatusBadRequest, errInvalidIfHeader
+	//	}
+	//	if len(ih.lists) == 1 && len(ih.lists[0].conditions) == 1 {
+	//		token = ih.lists[0].conditions[0].Token
+	//	}
+	//	if token == "" {
+	//		return http.StatusBadRequest, errInvalidLockToken
+	//	}
+	//	ld, err = ls.Refresh(now, token, duration)
+	//	if err != nil {
+	//		if err == ErrNoSuchLock {
+	//			return http.StatusPreconditionFailed, err
+	//		}
+	//		return http.StatusInternalServerError, err
+	//	}
+	//
+	//} else {
+	//	// Section 9.10.3 says that "If no Depth header is submitted on a LOCK request,
+	//	// then the request MUST act as if a "Depth:infinity" had been submitted."
+	//	depth := infiniteDepth
+	//	if hdr := r.Header.Get("Depth"); hdr != "" {
+	//		depth = parseDepth(hdr)
+	//		if depth != 0 && depth != infiniteDepth {
+	//			// Section 9.10.3 says that "Values other than 0 or infinity must not be
+	//			// used with the Depth header on a LOCK method".
+	//			return http.StatusBadRequest, errInvalidDepth
+	//		}
+	//	}
+	//	reqPath, status, err := h.stripPrefix(r.URL.Path, fs.User.ID)
+	//	if err != nil {
+	//		return status, err
+	//	}
+	//	ld = LockDetails{
+	//		Root:      reqPath,
+	//		Duration:  duration,
+	//		OwnerXML:  li.Owner.InnerXML,
+	//		ZeroDepth: depth == 0,
+	//	}
+	//	token, err = ls.Create(now, ld)
+	//	if err != nil {
+	//		if err == ErrLocked {
+	//			return StatusLocked, err
+	//		}
+	//		return http.StatusInternalServerError, err
+	//	}
+	//	defer func() {
+	//		if retErr != nil {
+	//			ls.Unlock(now, token)
+	//		}
+	//	}()
+	//
+	//	// Create the resource if it didn't previously exist.
+	//	//if _, err := h.FileSystem.Stat(ctx, reqPath); err != nil {
+	//	//	f, err := h.FileSystem.OpenFile(ctx, reqPath, os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0666)
+	//	//	if err != nil {
+	//	//		// TODO: detect missing intermediate dirs and return http.StatusConflict?
+	//	//		return http.StatusInternalServerError, err
+	//	//	}
+	//	//	f.Close()
+	//	//	created = true
+	//	//}
+	//
+	//	// http://www.webdav.org/specs/rfc4918.html#HEADER_Lock-Token says that the
+	//	// Lock-Token value is a Coded-URL. We add angle brackets.
+	//	w.Header().Set("Lock-Token", "<"+token+">")
+	//}
+	//
+	//w.Header().Set("Content-Type", "application/xml; charset=utf-8")
+	//if created {
+	//	// This is "w.WriteHeader(http.StatusCreated)" and not "return
+	//	// http.StatusCreated, nil" because we write our own (XML) response to w
+	//	// and Handler.ServeHTTP would otherwise write "Created".
+	//	w.WriteHeader(http.StatusCreated)
+	//}
 
-	} else {
-		// Section 9.10.3 says that "If no Depth header is submitted on a LOCK request,
-		// then the request MUST act as if a "Depth:infinity" had been submitted."
-		depth := infiniteDepth
-		if hdr := r.Header.Get("Depth"); hdr != "" {
-			depth = parseDepth(hdr)
-			if depth != 0 && depth != infiniteDepth {
-				// Section 9.10.3 says that "Values other than 0 or infinity must not be
-				// used with the Depth header on a LOCK method".
-				return http.StatusBadRequest, errInvalidDepth
-			}
-		}
-		reqPath, status, err := h.stripPrefix(r.URL.Path, fs.User.ID)
-		if err != nil {
-			return status, err
-		}
-		ld = LockDetails{
-			Root:      reqPath,
-			Duration:  duration,
-			OwnerXML:  li.Owner.InnerXML,
-			ZeroDepth: depth == 0,
-		}
-		token, err = h.LockSystem[fs.User.ID].Create(now, ld)
-		if err != nil {
-			if err == ErrLocked {
-				return StatusLocked, err
-			}
-			return http.StatusInternalServerError, err
-		}
-		defer func() {
-			if retErr != nil {
-				h.LockSystem[fs.User.ID].Unlock(now, token)
-			}
-		}()
-
-		// Create the resource if it didn't previously exist.
-		//if _, err := h.FileSystem.Stat(ctx, reqPath); err != nil {
-		//	f, err := h.FileSystem.OpenFile(ctx, reqPath, os.O_RDWR|os.O_CREATE|os.O_TRUNC, 0666)
-		//	if err != nil {
-		//		// TODO: detect missing intermediate dirs and return http.StatusConflict?
-		//		return http.StatusInternalServerError, err
-		//	}
-		//	f.Close()
-		//	created = true
-		//}
-
-		// http://www.webdav.org/specs/rfc4918.html#HEADER_Lock-Token says that the
-		// Lock-Token value is a Coded-URL. We add angle brackets.
-		w.Header().Set("Lock-Token", "<"+token+">")
-	}
-
-	w.Header().Set("Content-Type", "application/xml; charset=utf-8")
-	if created {
-		// This is "w.WriteHeader(http.StatusCreated)" and not "return
-		// http.StatusCreated, nil" because we write our own (XML) response to w
-		// and Handler.ServeHTTP would otherwise write "Created".
-		w.WriteHeader(http.StatusCreated)
-	}
-	writeLockInfo(w, token, ld)
+	writeLockInfo(w, fmt.Sprintf("%d", time.Now().UnixNano()), LockDetails{
+		Duration: duration,
+		OwnerXML: fs.User.Email,
+		Root:     reqPath,
+	})
 	return 0, nil
 }
 
 // OK
-func (h *Handler) handleUnlock(w http.ResponseWriter, r *http.Request, fs *filesystem.FileSystem) (status int, err error) {
+func (h *Handler) handleUnlock(w http.ResponseWriter, r *http.Request, fs *filesystem.FileSystem, ls LockSystem) (status int, err error) {
 	defer fs.Recycle()
+	return http.StatusNoContent, err
 
-	// http://www.webdav.org/specs/rfc4918.html#HEADER_Lock-Token says that the
-	// Lock-Token value is a Coded-URL. We strip its angle brackets.
-	t := r.Header.Get("Lock-Token")
-	if len(t) < 2 || t[0] != '<' || t[len(t)-1] != '>' {
-		return http.StatusBadRequest, errInvalidLockToken
-	}
-	t = t[1 : len(t)-1]
-
-	switch err = h.LockSystem[fs.User.ID].Unlock(time.Now(), t); err {
-	case nil:
-		return http.StatusNoContent, err
-	case ErrForbidden:
-		return http.StatusForbidden, err
-	case ErrLocked:
-		return StatusLocked, err
-	case ErrNoSuchLock:
-		return http.StatusConflict, err
-	default:
-		return http.StatusInternalServerError, err
-	}
+	//// http://www.webdav.org/specs/rfc4918.html#HEADER_Lock-Token says that the
+	//// Lock-Token value is a Coded-URL. We strip its angle brackets.
+	//t := r.Header.Get("Lock-Token")
+	//if len(t) < 2 || t[0] != '<' || t[len(t)-1] != '>' {
+	//	return http.StatusBadRequest, errInvalidLockToken
+	//}
+	//t = t[1 : len(t)-1]
+	//
+	//switch err = ls.Unlock(time.Now(), t); err {
+	//case nil:
+	//	return http.StatusNoContent, err
+	//case ErrForbidden:
+	//	return http.StatusForbidden, err
+	//case ErrLocked:
+	//	return StatusLocked, err
+	//case ErrNoSuchLock:
+	//	return http.StatusConflict, err
+	//default:
+	//	return http.StatusInternalServerError, err
+	//}
 }
 
 // OK
-func (h *Handler) handlePropfind(w http.ResponseWriter, r *http.Request, fs *filesystem.FileSystem) (status int, err error) {
+func (h *Handler) handlePropfind(w http.ResponseWriter, r *http.Request, fs *filesystem.FileSystem, ls LockSystem) (status int, err error) {
 	defer fs.Recycle()
 
 	reqPath, status, err := h.stripPrefix(r.URL.Path, fs.User.ID)
@@ -664,7 +680,7 @@ func (h *Handler) handlePropfind(w http.ResponseWriter, r *http.Request, fs *fil
 		}
 		var pstats []Propstat
 		if pf.Propname != nil {
-			pnames, err := propnames(ctx, fs, h.LockSystem[fs.User.ID], info)
+			pnames, err := propnames(ctx, fs, ls, info)
 			if err != nil {
 				return err
 			}
@@ -674,9 +690,9 @@ func (h *Handler) handlePropfind(w http.ResponseWriter, r *http.Request, fs *fil
 			}
 			pstats = append(pstats, pstat)
 		} else if pf.Allprop != nil {
-			pstats, err = allprop(ctx, fs, h.LockSystem[fs.User.ID], info, pf.Prop)
+			pstats, err = allprop(ctx, fs, ls, info, pf.Prop)
 		} else {
-			pstats, err = props(ctx, fs, h.LockSystem[fs.User.ID], info, pf.Prop)
+			pstats, err = props(ctx, fs, ls, info, pf.Prop)
 		}
 		if err != nil {
 			return err
@@ -699,7 +715,7 @@ func (h *Handler) handlePropfind(w http.ResponseWriter, r *http.Request, fs *fil
 	return 0, nil
 }
 
-func (h *Handler) handleProppatch(w http.ResponseWriter, r *http.Request, fs *filesystem.FileSystem) (status int, err error) {
+func (h *Handler) handleProppatch(w http.ResponseWriter, r *http.Request, fs *filesystem.FileSystem, ls LockSystem) (status int, err error) {
 	defer fs.Recycle()
 
 	reqPath, status, err := h.stripPrefix(r.URL.Path, fs.User.ID)
@@ -721,7 +737,7 @@ func (h *Handler) handleProppatch(w http.ResponseWriter, r *http.Request, fs *fi
 	if err != nil {
 		return status, err
 	}
-	pstats, err := patch(ctx, fs, h.LockSystem[fs.User.ID], reqPath, patches)
+	pstats, err := patch(ctx, fs, ls, reqPath, patches)
 	if err != nil {
 		return http.StatusInternalServerError, err
 	}
